package others;

import com.lti.civil.CaptureDeviceInfo;
import com.lti.civil.CaptureException;
import com.lti.civil.CaptureObserver;
import com.lti.civil.CaptureStream;
import com.lti.civil.CaptureSystem;
import com.lti.civil.CaptureSystemFactory;
import com.lti.civil.DefaultCaptureSystemFactorySingleton;
import com.lti.civil.Image;
import com.lti.civil.awt.AWTImageConverter;
import com.lti.civil.swing.ImageFrame;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.MediaTracker;
import java.awt.Toolkit;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.image.BufferedImage;
import java.awt.image.ImageObserver;
import java.awt.image.PixelGrabber;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;
import java.util.Vector;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JOptionPane;


/**
 * Simple GUI to display captured video.
 * @author Ali Karakaya
 *
 */
public class CaptureFrame extends ImageFrame
{
        static JFrame JF;
	public static void main(String[] args) throws CaptureException
	{               
		new CaptureFrame(DefaultCaptureSystemFactorySingleton.instance()).run();		
	}
        
	
	private CaptureSystem system;
	private CaptureStream captureStream;
	private final CaptureSystemFactory factory;
	private volatile boolean disposing = false;
	
	
	public CaptureFrame(CaptureSystemFactory factory)
	{	
		super("My Camera");
		this.factory = factory;
	
	}
	
	public void run() throws CaptureException
	{
		
		initCapture();
        //        setSize(200,200);
		setSize(captureStream.getVideoFormat().getWidth(), captureStream.getVideoFormat().getHeight());
                this.setDefaultCloseOperation(this.DO_NOTHING_ON_CLOSE);
		setLocation(0, 0);
		
		addWindowListener(new WindowAdapter()
		{
			public void windowClosing(WindowEvent e)
			{
				try
				{
					disposeCapture();
				} catch (CaptureException e1)
				{
					e1.printStackTrace();
				}
			//	System.exit(0);
			}
		});

		

		setVisible(true);
		pack();
		
		startCapture();
	}
	

	
	public void initCapture() throws CaptureException
	{
		
		system = factory.createCaptureSystem();
		system.init();
		List list = system.getCaptureDeviceInfoList();
		for (int i = 0; i < list.size(); ++i)
		{
			CaptureDeviceInfo info = (CaptureDeviceInfo) list.get(i);
			
			System.out.println("Device ID " + i + ": " + info.getDeviceID());
			System.out.println("Description " + i + ": " + info.getDescription());
			captureStream = system.openCaptureDeviceStream(info.getDeviceID());
			captureStream.setObserver((CaptureObserver) new MyCaptureObserver());
			
			break;
		}
		
	}
	
	public void startCapture() throws CaptureException
	{
		captureStream.start();
	}

	public void disposeCapture() throws CaptureException
	{
		disposing = true;
		
		if (captureStream != null)
		{	System.out.println("disposeCapture: stopping capture stream...");
			captureStream.stop();
			System.out.println("disposeCapture: stopped capture stream.");
			captureStream.dispose();
			captureStream = null;
		}
		
		if (system != null)
			system.dispose();
		System.out.println("disposeCapture done.");
                this.setVisible(false);
	}

	
	class MyCaptureObserver implements CaptureObserver
	{
        //    int sumNew=0,sumOld=0;
            double summation;
            int[] red=new int[3];
            int[] green=new int[3];
            int[] blue=new int[3];

            ArrayList<Integer> al=new ArrayList<Integer>();
		public void onError(CaptureStream sender, CaptureException e)
		{	
			e.printStackTrace();
		}

		public void onNewImage(CaptureStream sender, Image image)
		{	
			if (disposing)
				return;
                        
			try
			{
                           //     sumNew=0;
                                summation = 0.0;

				setImage(AWTImageConverter.toBufferedImage(image));
                                Graphics g = AWTImageConverter.toBufferedImage(image).createGraphics();
                                ImageIcon icon = new ImageIcon();
                                icon.setImage(AWTImageConverter.toBufferedImage(image));
                                BufferedImage bimg = new BufferedImage(320,320,BufferedImage.TYPE_INT_ARGB);
                                bimg=AWTImageConverter.toBufferedImage(image);
                                int[] rgbs = new int[320*320];
                                bimg.getRGB(0, 0, 320, 320, rgbs, 0, 320);
                                int r,gr,b;
 
 
                                
                                for(int i=0;i<320*320;i++)
                                {
                                r = (rgbs[i] >>> 16) & 0xFF;
                                gr = (rgbs[i] >>> 8) & 0xFF;
                                b = (rgbs[i]) & 0xFF;
                                summation+=r+gr+b;
                                }
                                /*
                                int a = (rgbs[10] >>> 24) & 0xFF;
                                red[0] = (rgbs[10] >>> 16) & 0xFF;
                                green[0] = (rgbs[10] >>> 8) & 0xFF;
                                blue[0] = (rgbs[10]) & 0xFF;

                                a = (rgbs[350] >>> 24) & 0xFF;
                                red[1] = (rgbs[350] >>> 16) & 0xFF;
                                green[1] = (rgbs[350] >>> 8) & 0xFF;
                                blue[1] = (rgbs[350]) & 0xFF;

                                a = (rgbs[320*319] >>> 24) & 0xFF;
                                red[2] = (rgbs[320*319] >>> 16) & 0xFF;
                                green[2] = (rgbs[320*319] >>> 8) & 0xFF;
                                blue[2] = (rgbs[320*319]) & 0xFF;
                                 *
                                 */
                               int changeDescripter = 0;
                               /*
                               for(int i=0;i<3;i++)
                               {
                                   sumNew+=red[i]+green[i]+blue[i];
                               }
                                *
                                */
                               /*
                                if(summation/(320.0*320.0*3)<30)
                                {
                                    if(changeDescripter!=1)
                                    {

                                    changeDescripter = 1;
                                    }
                                }
                                else if(summation/(320.0*320.0*3)<60)
                                {
                                    if(changeDescripter!=2)
                                    {

                                    changeDescripter = 2;
                                    }
                                }
                                else if(summation/(320.0*320.0*3)<120)
                                {
                                    if(changeDescripter!=3)
                                    {

                                    changeDescripter = 3;
                                    }
                                }
                                else if(summation/(320.0*320.0*3)<150)
                                {
                                    if(changeDescripter!=4)
                                    {

                                    changeDescripter = 4;
                                    }
                                }
                                else
                                {
                                   if(changeDescripter!=5)
                                    {

                                    changeDescripter = 5;
                                    }
                                }

                                /*
                               if(al.size()<10)
                                   al.add(sumNew);
                               else
                               {
                                    int old=al.get(0);
                                    if(sumNew-300>old)
                                    {
                                        silinecek.SilinecekView.ReDesignGUIDown();
                                        System.out.println("brighten detected");
                                    }
                                    if(sumNew+300<old)
                                    {
                                        silinecek.SilinecekView.ReDesignGUIUp();
                                        System.out.println("darkness detected");
                                    }

                                    al.remove(0);
                                    al.add(sumNew);
                               }
                               */
                           //    System.out.println(summation);
                          //     System.out.println("-----------");

			}
			catch (Throwable t)
			{	t.printStackTrace();
			}
		}
		
	}
}
