package others;


import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.media.*;
import javax.media.format.*;
import java.awt.*;

public class MotionDetectionEffect implements Effect {


  /**
   * Optimization. Anything above 0 turns it on. By default its
   * disabled.
   */
    public int OPTIMIZATION = 0;
    /**
     * Maximum threshold setting. Setting the threshold above this
     * means to get the motion detection to pass the frame you pretty
     * much have to full the whole frame with lots of motions (ie: drop
     * the camera)
     */
    public int THRESHOLD_MAX = 10000;

    /**
     * By what value you should increment.
     */
    public int THRESHOLD_INC = 1000;

    /**
     * The initial threshold setting.
     */
    public int THRESHOLD_INIT = 5000;

    private final static int INITIAL_SQUARE_SIZE = 5;

    public final static Format[] supportedFormat = new Format[] { //
    new RGBFormat(null, //
            Format.NOT_SPECIFIED, //
            Format.byteArray, //
            Format.NOT_SPECIFIED, //
            24, //
            3, 2, 1, //
            3, Format.NOT_SPECIFIED, //
            Format.TRUE, //
            Format.NOT_SPECIFIED) //
    };

    private Format inputFormat;

    private Format outputFormat;

    private Format[] inputFormats;

    private Format[] outputFormats;

    private int[] bwPixels;

    private byte[] bwData;

    private int oldMovement=0;

    /**
     * Visual mode is set.
     */
    private boolean visualize = true;

    /**
     * Server mode is set.
     */
    private boolean serverActive = true;

    /**
     * Update requested is set.
     */
    private boolean updateRequested;

    private int avg_ref_intensity;

    private int avg_img_intensity;

    /**
     * The RGBFormat of the inbuffer.
     */
    private RGBFormat vfIn = null;

    /**
     * Threshold multiplier in terms of gradient
     * High index means strong motion
     */

    private int[] ThresholdMultiplier = { 1, 3, 6, 12 };

    /**
     * Four different thresholds. Set initial values here.
     */
    private int[] threshs = { 20, 30, 40, 50 };

    private int det_thresh = threshs[1];

    /**
     *  The corresponding colours to the four different thresholds.
     */
    private int[] colors = { 0x00FF0000, 0x00FF9900, 0x00FFFF00, 0x00FFFFFF };

    /**
     *  The mean values of all squares in an image.
     */
    private int[] newImageSquares = null;

    /**
     *  The mean values of all squares in an image.
     */
    private int[] oldImageSquares = null;

    /**
     *  The difference of all the mean values of all squares in an image.
     */
    private int[] changedSquares = null;

    /**
     * The number of squares fitted in the image.
     */
    private int numberOfSquaresWide;

    /**
     * The number of squares fitted in the image.
     */
    private int numberOfSquaresHigh;

    /**
     * The number of squares fitted in the image.
     */
    private int numberOfSquares;

    /**
     * The square side, in pixels.
     */
    private int sqSide = INITIAL_SQUARE_SIZE;

    /**
     * The square area, in pixels.
     */
    private int sqArea = 0;

    /**
     * The amount of pixels left when all normal sized squares have been removed.
     */
    private int sqWidthLeftover = 0;

    /**
     * The amount of pixels left when all normal sized squares have been removed.
     */
    private int sqHeightLeftover = 0;

    /**
     * Optional, less processing is needed if some pixels are left out during some of the calculations.
     */
    private int pixelSpace = 0;

    /**
     * Image property.
     */
    private int imageWidth = 0;

    /**
     * Image property.
     */
    private int imageHeight = 0;

    /**
     * Image property.
     */
    private int imageArea = 0;



    private byte[] refData;

    private int widthOver3;
    private int heightOver3;
    private int rowPartition;
    private int columnPartition;
    private int region;
    private int[] regionSum = new int[9];
    private double allRegionWeightedSum;
    private int[] movementThreshold = { 15000 , 50000 };

    private double totalRGB=0;
    private int[] queue = new int[50];
    private int[] queue2 = new int[50];
    private int currentIndex=0;
    private int currentIndex2=0;
    /**
     * The threshold for determing if the pixel at a certain location has
     * changed consideribly.
     */
    public int threshold = 30;
    /**
     * Our threshold for determinig if the input image has enough motion.
     *
     */
    public int blob_threshold = THRESHOLD_INIT;

    /**
     * Turn debugging on. Slows down the effect but shows how motion
     * detection effect works.
     */
    public boolean debug = false;

    /**
     * Initialize the effect plugin.
     */
      public MotionDetectionEffect() {
        inputFormats = new Format[] { new RGBFormat(null, Format.NOT_SPECIFIED, Format.byteArray, Format.NOT_SPECIFIED, 24, 3, 2, 1, 3, Format.NOT_SPECIFIED, Format.TRUE, Format.NOT_SPECIFIED) };

        outputFormats = new Format[] { new RGBFormat(null, Format.NOT_SPECIFIED, Format.byteArray, Format.NOT_SPECIFIED, 24, 3, 2, 1, 3, Format.NOT_SPECIFIED, Format.TRUE, Format.NOT_SPECIFIED) };

    }

      /**
       * Initalize all regions to 0
       * _____________
       * | 4 | 2 | 4 |
       * -------------
       * | 2 | 1 | 2 |
       * -------------
       * | 1 | 0 | 1 |
       * -------------
       * Region Multiplication matrix
       */
    public void RegionFill() {
        for(int i=0;i<9;i++)
            regionSum[i]=0;
    }

    private void QueueFill() {
        for(int i=0;i<50;i++)
            queue[i]=-1;
    }

    private void QueueFill2() {
        for(int i=0;i<50;i++)
            queue2[i]=-1;
    }

    /**
     * Get the format that we support
     *
     * @return  instanceof RGBFormat
     */
    public Format[] getSupportedInputFormats() {
	return inputFormats;
    }

    /**
     * Get the format that we support
     *
     * @return  instanceof RGBFormat
     */
    public Format[] getSupportedOutputFormats(Format input) {
        if (input == null) {
            return outputFormats;
        }
        if (matches(input, inputFormats) != null) {
            return new Format[] { outputFormats[0].intersects(input) };
        } else {
            return new Format[0];
        }
    }

    /**
     * Set our input format.
     */
    public Format setInputFormat(Format input) {
        inputFormat = input;
        return input;
    }

    /**
     * Set our output format.
     *
     */
    public Format setOutputFormat(Format output) {

        if (output == null || matches(output, outputFormats) == null)
            return null;

        RGBFormat incoming = (RGBFormat) output;

        Dimension size = incoming.getSize();
        int maxDataLength = incoming.getMaxDataLength();
        int lineStride = incoming.getLineStride();
        float frameRate = incoming.getFrameRate();
        int flipped = incoming.getFlipped();
        int endian = incoming.getEndian();

        if (size == null)
            return null;
        if (maxDataLength < size.width * size.height * 3)
            maxDataLength = size.width * size.height * 3;
        if (lineStride < size.width * 3)
            lineStride = size.width * 3;
        if (flipped != Format.FALSE)
            flipped = Format.FALSE;

        outputFormat = outputFormats[0].intersects(new RGBFormat(size, maxDataLength, null, frameRate, Format.NOT_SPECIFIED, Format.NOT_SPECIFIED, Format.NOT_SPECIFIED, Format.NOT_SPECIFIED, Format.NOT_SPECIFIED, lineStride, Format.NOT_SPECIFIED, Format.NOT_SPECIFIED));
        QueueFill();
        QueueFill2();
        return outputFormat;
    }

    /**
     * Process the buffer. This is where motion is analysed and optionally visualized.
     *
     */

    public synchronized int process(Buffer inBuffer, Buffer outBuffer) {
        int outputDataLength = ((VideoFormat) outputFormat).getMaxDataLength();
        validateByteArraySize(outBuffer, outputDataLength);
        outBuffer.setLength(outputDataLength);
        outBuffer.setFormat(outputFormat);
        outBuffer.setFlags(inBuffer.getFlags());

        byte[] inData = (byte[]) inBuffer.getData();
        byte[] outData = (byte[]) outBuffer.getData();
        int[] sqAvg = null;
        int[] refsqAvg = null;

        vfIn = (RGBFormat) inBuffer.getFormat();
        Dimension sizeIn = vfIn.getSize();

        int pixStrideIn = vfIn.getPixelStride();
        int lineStrideIn = vfIn.getLineStride();

        imageWidth = (vfIn.getLineStride()) / 3; //Divide by 3 since each pixel has 3 colours.
        imageHeight = ((vfIn.getMaxDataLength()) / 3) / imageWidth;
        imageArea = imageWidth * imageHeight;

        int r, g, b = 0; //Red, green and blue values.
        RegionFill();
        

        if (oldImageSquares == null) { //For the first frame.
            changeSqSize(INITIAL_SQUARE_SIZE);
            updateRequested = true;
        }

        //Copy all data from the inbuffer to the outbuffer. The purpose is to display the video input on the screen.
        System.arraycopy(inData, 0, outData, 0, outData.length);

        // Simplify the image to black and white, image information shrinks to one third of the original amount. Less processing needed.
        bwPixels = new int[outputDataLength / 3];
//        System.out.println(outputDataLength);
//        System.out.println(imageWidth);
//        System.out.println(imageHeight);
        for (int ip = 0; ip < outputDataLength; ip += 3) {
            int bw = 0;
            r = (int) inData[ip] & 0xFF;
            g = (int) inData[ip + 1] & 0xFF;
            b = (int) inData[ip + 2] & 0xFF;

            bw = (int) ((r + b + g) / (double) 3);
            if( ((ip/3)%640<210 || (ip/3)%640>430) && (ip/3)%480<160)
                totalRGB+=bw;
            bwPixels[ip / 3] = bw; //Now containing a black and white image.
        }

        if (updateRequested) {
            updateRequested = false;
            updateSquares();
            return BUFFER_PROCESSED_OK;
        } else {
            updateSquares();
            oldNewChange();
            int c = 0;
            for (int i = 0; i < changedSquares.length; i++) {
                if (changedSquares[i] > det_thresh) {
                    c++;
                }
            }

            if (c > 10 && serverActive) {
                //    try{
                System.out.println("Motion detected (motion at " + c + "areas");
                //      multicast.send("Motion detected");  - Disabled
                //    } catch(IOException e){}
            }
//Focus here! Calculating flow with respect to the division must be there.
            // If chosen, the detected motion is presented on top of the video input, thus covering the edges of the moving object.
           /* System.out.println(numberOfSquares);
            System.out.println(numberOfSquaresWide);
            System.out.println(numberOfSquaresHigh);
            System.out.println(sqSide);
            System.out.println(imageWidth);*/
            if (visualize) {
                for (int i = 1; i <= numberOfSquares; i++) { // For all blobs
                    if ((changedSquares[i - 1] > threshs[0])) { // Critical threshold, if less, then no motion is said to have occured.
                        if (((i % numberOfSquaresWide) != 0) && (numberOfSquares - i) > numberOfSquaresWide) {//Normal square, the other cases is not presented!
                            int begin = ((((i % numberOfSquaresWide) - 1) * sqSide) + ((i / numberOfSquaresWide) * imageWidth * sqSide)) * 3; //Calculate start of square.
                            switch((i/numberOfSquaresWide)/heightOver3)
                            {
                                case 0:
                                    rowPartition=1;
                                    break;
                                case 1:
                                    rowPartition=2;
                                    break;
                                case 2:
                                    rowPartition=3;
                                    break;
                                default:
                           //         System.out.println("Row Partition Problem");
                                    rowPartition=3;
                                    break;
                            }

                            switch((i%numberOfSquaresWide)/widthOver3)
                            {
                                case 0:
                                    columnPartition=1;
                                    break;
                                case 1:
                                    columnPartition=2;
                                    break;
                                case 2:
                                    columnPartition=3;
                                    break;
                                default:
                           //         System.out.println("Column Partition Problem");
                                    columnPartition=3;
                                    break;

                            }

                            region=3*(rowPartition-1)+columnPartition;
              //              System.out.println(region);
                            if (changedSquares[i - 1] > threshs[3]) { //Very strong motion.
                                b = (byte) (colors[3] & 0xFF);
                                g = (byte) ((colors[3] >> 8) & 0xFF);
                                r = (byte) ((colors[3] >> 16) & 0xFF);
                                regionSum[region-1]+=ThresholdMultiplier[3];
                            } else if (changedSquares[i - 1] > threshs[2]) { //Strong motion.
                                b = (byte) (colors[2] & 0xFF);
                                g = (byte) ((colors[2] >> 8) & 0xFF);
                                r = (byte) ((colors[2] >> 16) & 0xFF);
                                regionSum[region-1]+=ThresholdMultiplier[2];
                            } else if (changedSquares[i - 1] > threshs[1]) { //Weak motion.
                                b = (byte) (colors[1] & 0xFF);
                                g = (byte) ((colors[1] >> 8) & 0xFF);
                                r = (byte) ((colors[1] >> 16) & 0xFF);
                                regionSum[region-1]+=ThresholdMultiplier[1];
                            } else { //The Weakest motion detected.
                                b = (byte) (colors[0] & 0xFF);
                                g = (byte) ((colors[0] >> 8) & 0xFF);
                                r = (byte) ((colors[0] >> 16) & 0xFF);
                                regionSum[region-1]+=ThresholdMultiplier[0];
                            }
                            for (int k = begin; k < (begin + (sqSide * imageWidth * 3)); k = k + (imageWidth * 3)) { //Ev <=
                                for (int j = k; j < (k + (sqSide * 3)); j = j + 3) {
                                    try {
                                        outData[j] = (byte) b;
                                        outData[j + 1] = (byte) g;
                                        outData[j + 2] = (byte) r;
                                    } catch (ArrayIndexOutOfBoundsException e) {
                                        System.out.println("Nullpointer: j = " + j + ". Outdata.length = " + outData.length);
                                        System.exit(1);
                                    }
                                }
                            }
                            
               //             System.out.println(regionSum[0]);
               //             System.out.println(regionSum[2]);
               //             System.out.println(allRegionWeightedSum);
                            allRegionWeightedSum=(regionSum[0]+regionSum[2])*4+(regionSum[1]+regionSum[3]+regionSum[5])*2+(regionSum[4]+regionSum[6]+regionSum[8]);
                           /*
                            if(allRegionWeightedSum<movementThreshold[0])
                                yaver10.Yaver10View.movementMode=0;
                            else if(allRegionWeightedSum <= movementThreshold[1] && regionSum[0]>200 && regionSum[2]>200)
                                yaver10.Yaver10View.movementMode=1;
                            else if(allRegionWeightedSum >= movementThreshold[1] && regionSum[0]>400 && regionSum[2]>400)
                                yaver10.Yaver10View.movementMode=2;
                            */
                            
               //             System.out.println(regionSum[0]);
               //             System.out.println(yaver10.Yaver10View.movementMode);
                            
                        }
                    }
                    
                }

                            System.out.println(regionSum[0]);
                            System.out.println(regionSum[2]);
                            System.out.println(allRegionWeightedSum);


                oldMovement=yaver10.Yaver10View.movementMode;
               // int newMovement;
                int i;
                for(i=0;i<49;i++)
                {
                    if(queue[i]!=queue[i+1])
                        break;

                }
                if(i==49)
                {
                    if(allRegionWeightedSum<movementThreshold[0])
                        yaver10.Yaver10View.movementMode=0;
                    else if(allRegionWeightedSum <= movementThreshold[1] && regionSum[0]>600 && regionSum[2]>1000)
                        yaver10.Yaver10View.movementMode=1;
                    else if(allRegionWeightedSum >= movementThreshold[1] && regionSum[0]>900 && regionSum[2]>1500)
                        yaver10.Yaver10View.movementMode=2;
                }
                
                queue[currentIndex]=yaver10.Yaver10View.movementMode;
                currentIndex++;
                if(currentIndex==50)
                    currentIndex=0;
                allRegionWeightedSum=0;
            }
        }




        int i;
        for(i=0;i<49;i++)
        {
            if(queue2[i]!=queue2[i+1])
            break;
        }
        if(i==49)
        {
           if(totalRGB<6000000)
            yaver10.Yaver10View.lightMode=0;
        else if(totalRGB<7100000)
            yaver10.Yaver10View.lightMode=1;
        else if(totalRGB<8200000)
            yaver10.Yaver10View.lightMode=2;
        else if(totalRGB<9000000)
            yaver10.Yaver10View.lightMode=3;
        else if(totalRGB<9500000)
            yaver10.Yaver10View.lightMode=4;
        }

                queue2[currentIndex2]=yaver10.Yaver10View.lightMode;
                currentIndex2++;
                if(currentIndex2==50)
                    currentIndex2=0;
        
        yaver10.Yaver10View.changeWRTLight();
        totalRGB=0.0;
        if(oldMovement!=yaver10.Yaver10View.movementMode && Messages.getWindows()[0]!=null)
        {
            Messages.CheckEntities();
        }
        if(oldMovement!=yaver10.Yaver10View.movementMode && Assignment.getWindows()[0]!=null)
        {
            Assignment.CheckEntities();
        }
        if(oldMovement!=yaver10.Yaver10View.movementMode)
        {
            try {
                yaver10.Yaver10View.CheckEntities();
            } catch (IOException ex) {
                Logger.getLogger(MotionDetectionEffect.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
        return BUFFER_PROCESSED_OK;
    }

    // methods for interface PlugIn
    public String getName() {
        return "Motion Detection Codec";
    }

    public void open() {
    }

    public void close() {
    }

    public void reset() {
    }

    public Object getControl(String controlType) {
        System.out.println(controlType);
        return null;
    }

    public Object[] getControls() {
        return null;
    }

    // Utility methods.
    public Format matches(Format in, Format outs[]) {
        for (int i = 0; i < outs.length; i++) {
            if (in.matches(outs[i]))
                return outs[i];
        }

        return null;
    }

    // methods for interface javax.media.Controls

    byte[] validateByteArraySize(Buffer buffer, int newSize) {
        Object objectArray = buffer.getData();
        byte[] typedArray;

        if (objectArray instanceof byte[]) { // Has correct type and is not null
            typedArray = (byte[]) objectArray;
            if (typedArray.length >= newSize) { // Has sufficient capacity
                return typedArray;
            }

            byte[] tempArray = new byte[newSize]; // Reallocate array
            System.arraycopy(typedArray, 0, tempArray, 0, typedArray.length);
            typedArray = tempArray;
        } else {
            typedArray = new byte[newSize];
        }

        buffer.setData(typedArray);
        return typedArray;
    }

    /**
     * Sets the current pixelspace, default is zero.
     * This is mainly for use where limited processing capacity are availible. Some pixels are left out in the calculations.
     * @param newSpace the space between two successive pixels.
     */
    private void setPixelSpace(int newSpace) {
        pixelSpace = newSpace;
    }

    private void changeSqSize(int newSide) {
        sqSide = newSide;
        sqArea = newSide * newSide;
        int wid = (imageWidth / sqSide); //The number of squares wide.
        int hei = (imageHeight / sqSide); //The number of squares high.
        sqWidthLeftover = imageWidth % sqSide;
        sqHeightLeftover = imageHeight % sqSide;
        if (sqWidthLeftover > 0) {
            wid++;
        }
        if (sqHeightLeftover > 0) {
            hei++;
        }

        numberOfSquaresWide = wid;
        numberOfSquaresHigh = hei;
        numberOfSquares = wid * hei;

        widthOver3=numberOfSquaresWide/3;
        heightOver3=numberOfSquaresHigh/3;

        newImageSquares = new int[numberOfSquares];
        oldImageSquares = new int[numberOfSquares];
        changedSquares = new int[numberOfSquares];
    }

    /**
     * Calculates the average colour in each square thus indirect eliminate noise.
     * @param startX the starting position of this square, in pixels, left edge.
     * @param startY the starting position of this square, in pixels, bottom edge.
     * @param sqWidth the width of this square, in pixels.
     * @param sqHeight the height of this square, in pixels.
     * @return The average greyscale value for this square.
     */
    private int averageInSquare(int startX, int startY, int sqWidth, int sqHeight) {
        int average = 0;
        for (int i = 0; i < sqHeight; i = i + 1 + pixelSpace) {// For all pixels
            for (int j = 0; j < sqWidth; j = j + 1 + pixelSpace) {
                average += bwPixels[(((startY + i) * imageWidth) + (startX + j))]; //Sum all the pixel values.
            }
        }
        average = average / (sqWidth * sqHeight); //Divide by the number of pixels to get the average value.
        return average;
    }

    /**
     * Backup the most recent frame examined. For the new frame, calculate the average greyscale value for all squares.
     */
    private void updateSquares() {
        System.arraycopy(newImageSquares, 0, oldImageSquares, 0, newImageSquares.length);
        int sqCount = 0; //Keep track of the current square
        for (int j = 0; j < (imageHeight); j = j + sqSide) { //For all squares
            for (int i = 0; i < (imageWidth); i = i + sqSide) {
                if (i <= (imageWidth - sqSide) && j <= (imageHeight - sqSide)) {
                    newImageSquares[sqCount] = averageInSquare(i, j, sqSide, sqSide); //No edge!
                } else if (i > (imageWidth - sqSide) && j <= (imageHeight - sqSide)) {
                    newImageSquares[sqCount] = averageInSquare(i, j, sqWidthLeftover, sqSide); //Right edge!
                } else if (i <= (imageWidth - sqSide) && j > (imageHeight - sqSide)) {
                    newImageSquares[sqCount] = averageInSquare(i, j, sqSide, sqHeightLeftover); //Bottom edge!
                } else if (i > (imageWidth - sqSide) && j > (imageHeight - sqSide)) {
                    newImageSquares[sqCount] = averageInSquare(i, j, sqWidthLeftover, sqHeightLeftover); //Bottom right edge!
                }
                sqCount++;
            }
        }
    }

    /**
     * Calculate the difference per square between currently stored frames.
     */
    private void oldNewChange() {
        for (int i = 0; i <= (numberOfSquares - 1); i++) { //For all squares
            int difference = Math.abs((newImageSquares[i]) - (oldImageSquares[i])); //Compare each square with the corresponding square in the previous frame.
            changedSquares[i] = difference; //Save the difference.
        }
    }


    void sample_down(byte[] inData, byte[] outData, int X, int Y, int width,
    int height, int lineStrideIn, int pixStrideIn) {

          int p1, p2, p3, p4, op,x,y;

      for ( y = 0; y < (height/2); y++) {

        p1 = (y * 2) * lineStrideIn ; // upper left cell
        p2 = p1 + pixStrideIn;                    // upper right cell
        p3 = p1 + lineStrideIn;         // lower left cell
        p4 = p3 + pixStrideIn;                    // lower right cell
        op = lineStrideIn * y + (lineStrideIn*Y) + (X*pixStrideIn);
        for ( int i =0; i< (width /2 );i++) {
/*
          outData[op++] = (byte)((inData[p1++] + inData[p2++] + inData[p3++] + inData[p4++])/4); // blue cells avg
          outData[op++] = (byte)((inData[p1++] + inData[p2++] + inData[p3++] + inData[p4++])/4); // green cells avg
          outData[op++] = (byte)((inData[p1++] + inData[p2++] + inData[p3++] + inData[p4++])/4); // red cells avg
*/
          outData[op++] = (byte)(((int)(inData[p1++] & 0xFF) +
            ((int)inData[p2++] & 0xFF)+ ((int)inData[p3++] & 0xFF) +
            ((int)inData[p4++] & 0xFF))/4); // blue cells avg
          outData[op++] = (byte)(((int)(inData[p1++] & 0xFF) +
            ((int)inData[p2++] & 0xFF)+ ((int)inData[p3++] & 0xFF) +
              ((int)inData[p4++] & 0xFF))/4); // blue cells avg
          outData[op++] = (byte)(((int)(inData[p1++] & 0xFF) +
            ((int)inData[p2++] & 0xFF)+ ((int)inData[p3++] & 0xFF) +
              ((int)inData[p4++] & 0xFF))/4); // blue cells avg
          p1 += 3; p2 += 3; p3+= 3; p4 += 3;
        }

      }
    }
}

   
